#include <Wire.h>
#include "n64.h"
#include "wii.h"

int error;
int oldError;
unsigned char dataFormat;
unsigned char controllerType[6];
extern bool connected;

int ADDRESS = 0x52;

    unsigned char conTypes[3][6]={
      {0x00,0x00,0xA4,0x20,0x00,0x00}, // 0 = Nunchuck
      {0x00,0x00,0xA4,0x20,0x01,0x01}, // 1 = Classic controller
      {0x01,0x00,0xA4,0x20,0x01,0x01}, // 2 = Classic controller pro
    };
    char *controller[3] = {"Nunchuck","Classic controller","Classic controller pro"};


//Common structure for last read data
uint8_t _data[21];
int _LX, _RX, _LY, _RY;
bool _RIGHT, _DOWN, _L, _MINUS, _HOME, _PLUS, _R, _LZ, _B, _Y, _A, _X, _RZ, _LEFT, _UP;

uint8_t wii_stick(unsigned int stick){
  return _data[stick];
}

void checkController(){
  Serial.print("Controller data format:");
  Serial.println(dataFormat);

  char controlType=-1;
  for(char s=0; s<3; s++){
    unsigned char tempCount=0;
    for(char t=0; t<6; t++){
      // check controller against known models. 5th byte ignored, as it is changed by init process
      if(t!=4){ if(controllerType[t] == conTypes[s][t]){ tempCount++; } }
    }
    if(tempCount==5){controlType=s;}
  }
  Serial.print("Controller:");
  if(controllerType<0){
    Serial.println("Not Connected");
  }else{
    Serial.println(controller[controlType]);
  }
  
}
void _sendByte(unsigned char data, unsigned char location)
{
  Wire.beginTransmission(ADDRESS);
  Wire.write(location);
  Wire.write(data);
  Wire.endTransmission();
}

void _write_zero(){
  Wire.beginTransmission(ADDRESS);      // transmit to device (byte)0x52
  Wire.write((byte)0x00);           // writes one byte
  Wire.endTransmission();    // stop transmitting
}

void update_classic(){
  int count = 0;
  int values[8];
  Serial.print("update_classic,");
  Wire.beginTransmission(ADDRESS);
  oldError=error;
  Serial.print(oldError);
  Serial.print(",");
  error=Wire.endTransmission();
  Serial.println(error);

  
  // if previous read was an error and current read is not, then we probably plugged on a controller, so init it.
  if(error==0 && oldError!=0){
    Serial.print("Init Classic");
    init_classic();
  }
  if(error!=0){
    connected=0;
    // if failed to connect, reset controller type
    for(char t=0; t<6; t++){
      controllerType[t] = 0x00;
    }
  }
  count=0;
 Wire.requestFrom(ADDRESS, 8);
  while(Wire.available()){
    _data[count] = Wire.read();
    count++;
  }

  if(dataFormat==3){ // NES Classic format
    _LX = _data[0];
    _RX = _data[1];
    _LY = _data[2];
    _RY = _data[3];

//    WiiController::lt = values[4];
//    WiiController::rt = values[5];

    _RIGHT = _data[6]&B10000000 ? 0:1;
     _DOWN = _data[6]&B01000000 ? 0:1;
        _L = _data[6]&B00100000 ? 0:1;
    _MINUS = _data[6]&B00010000 ? 0:1;
     _HOME = _data[6]&B00001000 ? 0:1;
     _PLUS = _data[6]&B00000100 ? 0:1;
        _R = _data[6]&B00000010 ? 0:1;

       _LZ = _data[7]&B10000000 ? 0:1;
        _B = _data[7]&B01000000 ? 0:1;
        _Y = _data[7]&B00100000 ? 0:1;
        _A = _data[7]&B00010000 ? 0:1;
        _X = _data[7]&B00001000 ? 0:1;
       _RZ = _data[7]&B00000100 ? 0:1;
     _LEFT = _data[7]&B00000010 ? 0:1;
       _UP = _data[7]&B00000001 ? 0:1;
  }
  
  _write_zero();
}


void init_classic()
{
  Wire.begin();
  _sendByte(0x55, 0xF0); // init controller
  _sendByte(0x00, 0xFB);
  _sendByte(0x03, 0xFE); // request datatype 3

  // check to see if the format is selected
  Wire.beginTransmission(ADDRESS);      // transmit to device (byte)0x52
  Wire.write((byte)0xFA);           // writes one byte
  Wire.endTransmission();    // stop transmitting
  delay(10); // needed for official classic controller
  Wire.requestFrom(ADDRESS, 6);
  char t=0;
  while(Wire.available()){
    controllerType[t++] = Wire.read();
  }
  dataFormat = controllerType[4];
  connected=1;

/*
  Known data types
  1. Standard Wii Classic controller
  2. ?
  3. Classic controller, easier to read, not universally supported by 3rd part controllers
  4. Wii Motion Plus
*/
  update_classic();
}

